import xlrd
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from pylab import *

"""
洋流沿纬度方向，从西向东
探究经度方向、正向，即(经、正)
    参数：
        1.表名: 输入全路径,带后缀
        2.Sheet索引: 从0开始
"""
def latitudePlus(name, sheet_index):
    Latitude,Longitude,POC_PON = read(name,sheet_index,1)
    # 根据探究目的划分列表
    list = latitudeOrientStream(Longitude, POC_PON)
    # 排序
    list.sort(key=lambda list: list[0])
    # x轴等间距
    name_nointerval,data_nointerval = noInterval(list,1)
    # 作图
    # 可选择重载函数，drawWithYMax(name_nointerval,data_nointerval,xlabel,ymax):
    # 用于指定y轴相同时使用
    draw(name_nointerval,data_nointerval,"经度")

# 洋流沿纬度方向，从东向西:
# 经、负
def latitudeMinus(name, sheet_index):
    Latitude, Longitude, POC_PON = read(name, sheet_index,0)
    list = latitudeOrientStream(Longitude, POC_PON)
    list.sort(key=lambda list: list[0], reverse=True)
    name_nointerval,data_nointerval = noInterval(list,0)
    draw(name_nointerval,data_nointerval,"经度")


# 洋流沿经度方向，从南向北
# 纬、正
def longitudePlus(name, sheet_index):
    Latitude, Longitude, POC_PON = read(name, sheet_index,1)
    list = longitudeOrientStream(Latitude, POC_PON)
    list.sort(key=lambda list: list[0])
    name_nointerval,data_nointerval = noInterval(list,1)
    draw(name_nointerval,data_nointerval,"纬度")


# 洋流沿经度方向，从北向南：
# 纬、负
def longitudeMinus(name, sheet_index):
    Latitude, Longitude, POC_PON = read(name, sheet_index,0)
    list = longitudeOrientStream(Latitude, POC_PON)
    list.sort(key=lambda list: list[0], reverse=True)
    name_nointerval,data_nointerval = noInterval(list,0)
    draw(name_nointerval, data_nointerval, "纬度")

# 读取excel,并预处理
# flag = 0 代表负，为1代表正
def read(name, sheet_index,flag):
    wb_read = xlrd.open_workbook(name)
    # 读取洋流
    ocean_current = wb_read.sheet_by_index(sheet_index)
    Latitude = ocean_current.col_values(0)[1:]
    Longitude = ocean_current.col_values(1)[1:]
    POC_PON = ocean_current.col_values(6)[1:]
    Depth = ocean_current.col_values(13)[1:]
    # LLP是Latitude、Loingtitude、POC_PON
    LLP = []
    # 筛选深度三百米以上的
    for i in range(0, len(Latitude) - 2):
        if (Depth[i] < 300):
            LLP.append([Latitude[i], Longitude[i], POC_PON[i]])
    # 排序并重新划分
    if(flag == 0):
        LLP.sort(key=lambda LLP: LLP[1], reverse=True)
    elif(flag == 1):
        LLP.sort(key=lambda LLP: LLP[1])
    Latitude = []
    Longitude = []
    POC_PON = []
    for i in range(0, len(LLP)):
        Latitude.append(LLP[i][0])
        Longitude.append(LLP[i][1])
        POC_PON.append(LLP[i][2])
    return Latitude, Longitude, POC_PON

"""
洋流沿经度方向
    参数：
        1.Lat 输入纬度
        2.POC_PON值
    返回值：
        返回形如[['纬度1',POC_PON值1,POC_PON值2],...]
"""
def longitudeOrientStream(Lat, POC_PON):
    result = [[int(Lat[0])]]
    i = 0
    j = 1
    team = 0
    while (i < len(Lat) - 2):
        result[team].append(POC_PON[i])
        if (j < len(Lat) - 3):
            if (int(Lat[i]) == int(Lat[j])):
                i += 1
                j += 1
            else:
                result.append([int(Lat[j])])
                team += 1
                i = j
                j += 1
        else:
            result[team].append(POC_PON[j])
            break
    return result


"""
洋流沿纬度方向
    参数：
        1.Lat 输入经度
        2.POC_PON值
    返回值：
        返回形如[['经度1',POC_PON值1,POC_PON值2],...]
"""
def latitudeOrientStream(Long, POC_PON):
    result = [[int(Long[0])]]
    i = 0
    j = 1
    team = 0
    while (i < len(Long) - 2):
        result[team].append(POC_PON[i])
        if (j < len(Long) - 3):
            if (int(Long[i]) == int(Long[j])):
                i += 1
                j += 1
            else:
                result.append([int(Long[j])])
                team += 1
                i = j
                j += 1
        else:
            result[team].append(POC_PON[j])
            break
    return result

# 生成等间距间隔图，让经度或纬度间隔相等
# flag = 0 为负，1为正
def noInterval(list,flag):
    data = []
    if flag == 0:
        name = [list[0][0] + 1]
    elif flag == 1:
        name = [list[0][0] - 1]
    for i in range(0, len(list)):
        name.append(list[i][0])
        data.append(list[i][1:])
    # 让经度或纬度间隔相等
    name_nointerval = []
    temp = name[0]
    for i in range(0, abs(name[-1] - name[0]) + 1):
        name_nointerval.append(temp)
        if flag == 1:
            temp += 1
        elif flag == 0:
            temp -= 1
    data_nointerval = []
    flag = True
    for i in range(1, len(name_nointerval)):
        for j in range(1, len(name)):
            if (name_nointerval[i] == name[j]):
                data_nointerval.append(data[j - 1])
                flag = False
                break
        if (flag == True):
            data_nointerval.append([0])
        else:
            flag = True
    return name_nointerval,data_nointerval

#作图
def draw(name_nointerval,data_nointerval,xlabel):
    rcParams['axes.unicode_minus'] = False
    rcParams['font.sans-serif'] = ['Simhei']
    plt.figure(figsize=(10, 8), dpi=80)
    # 注：如果需要让y轴相同，在此设置
    # plt.ylim(0, 20)
    plt.boxplot(data_nointerval, notch=False, patch_artist=False, boxprops={'color': 'black', 'linewidth': '2.0'},
                capprops={'color': 'black', 'linewidth': '2.0'}, showfliers=False)
    plt.xlabel(xlabel)
    plt.ylabel('POC_PON')
    plt.xticks(np.arange(len(name_nointerval)), name_nointerval)
    plt.show()

# 重载函数，用于指定y轴相同时使用
def drawWithYMax(name_nointerval,data_nointerval,xlabel,ymax):
    rcParams['axes.unicode_minus'] = False
    rcParams['font.sans-serif'] = ['Simhei']
    plt.figure(figsize=(10, 8), dpi=80)
    # 注：如果需要让y轴相同，在此设置
    plt.ylim(0, ymax)
    plt.boxplot(data_nointerval, notch=False, patch_artist=False, boxprops={'color': 'black', 'linewidth': '2.0'},
                capprops={'color': 'black', 'linewidth': '2.0'}, showfliers=False)
    plt.xlabel(xlabel)
    plt.ylabel('POC_PON')
    plt.xticks(np.arange(len(name_nointerval)), name_nointerval)
    plt.show()

"""
使用说明:
    1.四大主函数:
    (1)latitudePlus   -> 处理洋流流向沿纬度方向, 由西向东
    (2)latitudeMinus  -> 处理洋流流向沿纬度方向, 由东向西
    (3)longitudePlus  -> 处理洋流流向沿经度方向, 由南向北
    (4)longitudeMinus -> 处理洋流流向沿经度方向, 由北向南
    
    2.参数
    (1)文件全路径
    (2)Sheet索引
    
    3.Demo
    注:
    (1)东澳大利亚洋流沿经向，由北向南
        longitudeMinus("数据点按洋流分类.xls",0)
"""
longitudeMinus("数据点按洋流分类.xls",0)
longitudePlus("数据点按洋流分类.xls",2)